/***************************************************************************
 *   Copyright (C) 2004-2008 by Roberto Virga                              *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSBOINCDATA_H
#define KBSBOINCDATA_H

#include <QDateTime>
#include <QDomElement>
#include <QHash>
#include <QList>
#include <QString>
#include <QStringList>

#include <KDE/KUrl>

#include <kbscore_export.h>

/**
 * @brief BOINC-related functions
 *
 * Functions for converting or displaying output generated by the BOINC client.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
namespace KBSBOINC
{
  /**
   * @brief Conversion from UNIX time
   *
   * Converts time expressed as the number of seconds elapsed from 12:00 AM UTC
   * of Jan 1, 1970 to a QDateTime object, adjusting it to the local timezone.
   *
   * @param d the number of seconds since the epoch, stored as a double
   * @return the time as a QDateTime object, adjusted to the local timezone
   *
   * @sa formatUNIXDate(const QDateTime &date)
   */
  KBSCORE_EXPORT QDateTime parseUNIXDate(double d);
  
  /**
   * @brief Conversion to UNIX time
   *
   * Adjusts the local time (passed as a QDateTime object) to UTC, and returns
   * it as the number of seconds elapsed from 12:00 AM UTC of Jan 1, 1970.
   *
   * @param date the local time as a QDateTime object
   * @return the number of seconds since the epoch, adjusted to UTC
   *
   * @sa parseUNIXDate(double d)
   */
  KBSCORE_EXPORT double formatUNIXDate(const QDateTime &date);
  
  /**
   * @brief User-friendly representation of size
   *
   * Formats a size value, expressed in bytes, into a user-friendly
   * representation. The size will be given as number of KBs (respectively:
   * MBs, GBs, TBs) if the number is bigger than and divisible by 2^10
   * (respectively: 2^20, 2^30, 2^40).
   *
   * @param bytes the number of bytes.
   * @return a user-friendly representation.
   *
   * @author Roberto Virga <rvirga@users.sourceforge.net>
   */
  KBSCORE_EXPORT QString formatBytes(double bytes);
  
  /**
   * @brief User-friendly representation of time elapsed
   *
   * Formats a time, expressed the number of seconds elapsed, into a
   * user-friendly representation. The time will be given in a format similar
   * to QTime.toString(), i.e. HH:MM:SS, but where the hour field has variable
   * length.
   *
   * @param secs the number of seconds elapsed
   * @return A user-friendly representation
   */
  KBSCORE_EXPORT QString formatTime(double secs);
  
  /**
   * @brief Internal BOINC project name
   *
   * Converts the project master page URL into the name BOINC uses internally
   * to identify the project. For most projects, this is tipically just the
   * hostname; if the URL contains a path, however, it is added after the
   * hostname, with underscores in places of slashes.
   *
   * @param url project master page URL
   * @return the internal identifier for the project
   */
  KBSCORE_EXPORT QString parseProjectName(const KUrl &url);
}

/**
 * @brief Host information
 *
 * The type of information BOINC collects about the participating host computer
 * can be divided into three categories. <BR>
 * The first category comprises information about the hardware and operating
 * system. This will be obtained at startup by issuing the appropriate
 * operating system calls (different for each platform). To this class belongs
 * the time zone in which the computer resides (#timezone). Basic network
 * information, such as the domain name (#domain_name) and IP address
 * (#ip_addr). The name (#name) and version (#version) of the operating system.
 * The number of "CPUs" (#ncpus); BOINC does not distinguish between CPUs and
 * CPU cores; each multi-core CPU will be represented as many "CPUs" as its
 * cores. BOINC will also record the manufacturer (#vendor) and model (#model)
 * of these "CPUs", as well as the size of their internal cache (#cache).
 * Finally, it will extract some basic information about the available memory,
 * both physical (#nbytes) and virtual (#swap), as well as the storage space
 * size (#total) and the amount of it that is still available (#free). All
 * this info will be used to determine if the host satisfies the eligibility
 * requirement to run each available workunit. For example, if a workunit
 * requires 1GB of disk space, and the host has only 800M free, that workunit
 * won't be assigned to the host. <BR>
 * A second category of host information is instead obtained by periodically
 * benchmarking the computer. At regular intervals, BOINC suspends all activity
 * to run benchmarks whose objective is to compute the speed of the computer,
 * expressed as the number of floating point (#fpops) and integer (#iops)
 * operations that it is capable to execute per second, as well as the memory
 * bandwidth (#membw). These benchmarks don't measure just the speed, but also
 * the correctness of the computation; eventual errors in the computation are
 * also recorded, according to their type: floating point errors (#fpop_err),
 * integer errors (#iop_err), or errors while computing the memory bandwidth
 * (#membw_err). Information about the speed of the computer is used to`
 * compute the amount of computation performed, and therefore the credit that
 * the host will claim after having processed a workunit to completion.<BR>
 * The third class of data is not collected, but rather generated. For every
 * new host it encounters, BOINC generates a unique ID (#host_cpid) that it
 * subsequently uses to identify that computer among millions of others.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCHostInfo
{
  unsigned timezone;                                    ///< local timezone
  QString domain_name,                                  ///< domain name
          ip_addr,                                      ///< IP address
          host_cpid;                                    ///< host ID
  struct {
    unsigned ncpus;                                     ///< number of "CPUs"
    QString vendor,                                     ///< CPU vendor ('vendor' string reported by CPU)
            model;                                      ///< CPU model ('model' string reported by CPU)
    double fpops,                                       ///< floating point operations per second
           iops,                                        ///< integer operations per second
           membw;                                       ///< memory bandwith (in bytes/sec)
    unsigned fpop_err,                                  ///< floating point errors so far
             iop_err,                                   ///< integer errors so far
             membw_err;                                 ///< memory bandwith errors so far
    double calculated;                                  ///< benchmarked performance
  } p;                                                  ///< processor information
  struct {
    QString name,                                       ///< operating system name
            version;                                    ///< operating system version
  } os;                                                 ///< operating system information
  struct {
    double nbytes,                                      ///< RAM memory
           cache,                                       ///< cache memory
           swap;                                        ///< swap memory
  } m;                                                  ///< memory information
  struct {
    double total,                                       ///< total disk space
           free;                                        ///< free disk space
  } d;                                                  ///< disk information
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCHostInfo();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about the host computer.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
  
  /**
   * @brief Credit per CPU second
   *
   * Number of cobblestones (the unit measure of credit used by BOINC) per
   * second of this machine. Multiplied by the number of seconds a workunit
   * took to compute it, it gives the total credit that will be claimed for the
   * workunit. <BR>
   * This quantity depends solely on the number of floating point and integer
   * speed of the machine.
   *
   * @return amount of credit to be claimed for each second of work
   *
   * @sa credit_per_cpu_sec(double fpops, double iops)
   */
  double credit_per_cpu_sec() const;
  
  /**
   * @brief Credit per CPU second
   *
   * Number of cobblestones (the unit measure of credit used by BOINC) per
   * second of this machine. Multiplied by the number of seconds a workunit
   * took to compute it, it gives the total credit that will be claimed for the
   * workunit. <BR>
   * This quantity depends solely on the number of floating point and integer
   * speed of the machine, which are passed in input.
   *
   * @param fpops floating point operations per second
   * @param iops integer operations per second
   * @return amount of credit to be claimed for each second of work
   *
   * @sa credit_per_cpu_sec()
   */
  static double credit_per_cpu_sec(double fpops, double iops);
};

/**
 * @brief Time-related statistics
 *
 * BOINC collects some statistics about the percentage of time some of
 * the computer resources has been available to it. <BR>
 * Specifically, it computers the percentage of time BOINC has been running
 * with respect to the time the machine has been up (#on_frac). It also keeps
 * track of the percentage of time when, while running, it has performed some
 * real work (#active_frac) and has been able to connect to the network
 * (#connected_frac), recording the time of last connection
 * (#last_update). <BR>
 * Finally, it keeps track of what percentage of CPU cycles gets assigned
 * to it when it's doing work (#cpu_efficiency). On a computer used solely
 * for BOINC, or used also for light tasks (email, word processing), this
 * value will be very close to 1. On a computer also used for CPU-intensive
 * tasks, like gaming, this might be lower than 0.5.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCTimeStats
{
  double on_frac,                                       ///< percentage of time BOINC is running
         connected_frac,                                ///< percentage of time BOINC is connected to the network
         active_frac,                                   ///< percentage of time BOINC doing work (i.e. not suspended)
         cpu_efficiency;                                ///< percentage of the CPU assigned to BOINC while doing work
  QDateTime last_update;                                ///< date & time of last network connection
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCTimeStats();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains the
   * time-related statistics.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Network statistics
 *
 * The BOINC client keeps track of the average network bandwidth for outbound
 * transmissions (#bwup) and incoming ones (#bwdown). These values are updated
 * after every network communication, and are used by BOINC to provide
 * completion time estimates for file transfers.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCNetStats
{
  double bwup,                                          ///< upload bandwidth (in bytes/sec)
         bwdown;                                        ///< download bandwidth (in bytes/sec)
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCNetStats();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains the network
   * statistics.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Credit
 *
 * In BOINC, work done is rewarded by credit, expressed as cobblestones.
 * Credit granted is not necessarily an integer, but may, and tipically
 * will, have a fractional part. <BR>
 * For both the host and for the user associated to this host, BOINC will keep
 * track of the day and time the host (respectively: the user) first joined the
 * project, the total credit awarded, and the average daily credit. <BR>
 * Note that joining a project in the case of a user means creating an account,
 * while a host is automatically joined the first time that it contacts the
 * project scheduler requesting some work.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCCredits
{
  double total_credit,                                  ///< total credit
         expavg_credit;                                 ///< average daily credit
  QDateTime create_time;                                ///< date & time it joined the project
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCCredits();
};

/**
 * @brief Project
 *
 * The master URL (#master_url) of a project is its most fondamental resource.
 * Besides being used as unique identifier for the project, it provides to the
 * scheduler and data servers locations. This mechanism gives the project
 * absolute freedom in relocating their servers; as long as the master page
 * remains the same, users don't have to be notified of the change of location.
 * Second in importance to the master URL is the scheduler one
 * (#scheduler_url). The project scheduler is the entity which distributes
 * workunits and collects results. These operations are conducted through a
 * RPC mechanism. <BR>
 * To contribute to a project, a user must create an account. He/she will have
 * to provide some basic information, like a valid email address (#email_hash),
 * and choose an account name (#user_name). Optionally he can also join a team
 * (#team_name) of other participating users with similar interests. Once
 * joined the project, a cross-project identifier (#cross_project_id) will be
 * created for the user (if this is the first BOINC project he/she participates
 * in), or retrieved (if he/she already participates to other BOINC projects,
 * and therefore he/she already has one); this identifier will be used by BOINC
 * to compute some statistics about user participation in its projects. <BR>
 * User contributions will be measured by credits. A project keeps track not
 * only of the user's overall credits (#user), but also of each host used to
 * participate (#host). <BR>
 * When participating to multiple projects, the user can decide how much
 * CPU to give to each individual project by setting its resource share value
 * (#resource_share). The fraction of CPU time given to the project will then
 * be computed by dividing its resource share by the sum of all the shares.
 * <BR>
 * Aside from lowering the resource share, there are other two ways a user can
 * diminsh his/her participation in a project. A first way is to suspend it
 * (#is_suspended); this will put on hold the computation of all results
 * associated with the project. The second way is to request that no new
 * workunit to be downloaded from the project (#dont_request_more_work); this
 * is a clean way to withdraw from a project without leaving any outstanding
 * work undone.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCProject
{
  KUrl scheduler_url,                                   ///< scheduler URL
       master_url;                                      ///< project master page URL
  QString project_name,                                 ///< user-friendly name for the project
          user_name,                                    ///< name of the user (relative to this project)
          team_name,                                    ///< name of the team the user belongs to
          email_hash,                                   ///< obfuscated user email address
          cross_project_id;                             ///< unique ID to identify this user across multiple projects
  KBSBOINCCredits user;                                 ///< user credits
  unsigned rpc_seqno,                                   ///< RPC sequence number
           hostid;                                      ///< identifies this host in RPC communications
  KBSBOINCCredits host;                                 ///< host computer credits
  struct {
    double cpu,                                         ///< deprecated
           mod_time;                                    ///< deprecated
  } exp_avg;                                            ///< deprecated
  unsigned nrpc_failures,                               ///< number of failed RPC with scheduler
           master_fetch_failures;                       ///< number of failed fetches from master page
  QDateTime min_rpc_time;                               ///< earliest time a RPC can be done
  double short_term_debt,                               ///< used to determine when the project should get CPU time
         long_term_debt,                                ///< used to determine when the project should get new workunits
         resource_share;                                ///< used to compute how much CPU time to give to the project
  bool suspended_via_gui,                               ///< all project activity has been suspended
       dont_request_more_work,                          ///< do not download any more workunits
       attached_via_acct_mgr;                           ///< added by the account manager rather than manually by the user
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCProject();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a project.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief File information
 *
 * BOINC keeps some accounting information on all files it manages, both input
 * files downloaded from the project servers, and output files that it creates
 * as part of the computation of a result. The information stored, however,
 * varies slightly for these two types of files. <BR>
 * For both input and output files, it keeps track of the file name (#name),
 * and its current size. <BR>
 * For output files only, a project can specify a size bound (#max_nbytes); if
 * the file size exceeds this bound, the file is erased and its computation
 * aborted. For input files, this field has no purpose and it is always set to
 * 0. <BR>
 * A file has a status flag, that will be set to 1 if the file exists, 0 if
 * it doesn't (yet), and that will have negative value if there was an error
 * during the creation of the file. This error will typically be for input
 * files a network or I/O error in downloading and saving the data, and for
 * output files a violiation of the size bound mentioned before. <BR>
 * Finally, BOINC will associate a URL to each file. For input files, this
 * will be the location on the project data server from which it was
 * downloaded; for output files, it will instead be the address of the CGI
 * script that will be used to upload it to server.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCFileInfo
{
  QString name;                                         ///< file name, as stored in the filesystem
  double nbytes,                                        ///< current size (in bytes)
         max_nbytes;                                    ///< maximum size (in bytes) if created by BOINC, 0 otherwise
  int status;                                           ///< file status: 0 if not present, 1 if present, negative if error
  QList<KUrl> url;                                      ///< file URLs
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCFileInfo();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a specific file.
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Application
 *
 * In BOINC, an application stands for a family of one or more application
 * versions. For each application, we store its internal name (#name), an
 * optional, more intuitive nickname (#user_friendly_name), and the project
 * it belongs to (#project_master_url).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCApp
{
  QString name,                                         ///< internal name of the application
          user_friendly_name;                           ///< user-friendly name for the application
  KUrl project_master_url;                              ///< project master page URL
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a project client application.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief File reference
 *
 * A file reference allows BOINC to translate between d
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCFileRef
{
  QString file_name,                                    ///< file name, as stored in the file system
          open_name;                                    ///< internal name of the file
  bool main_program;                                    ///< it is a client application
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCFileRef();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains a file
   * reference.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Application version
 *
 * A BOINC project, during its lifetime, will probably release several versions
 * of a client application. This might be to fix errors, or to speedup or
 * to expand its functions. For consistency in the evaluation of results,
 * whenever a new application version is released, it is important that all the
 * workunit that were sent out before this new release still get computed using
 * the old version. Hence the BOINC client must be capable of keeping track
 * of multiple application versions, and make sure that each workunit is
 * computed using the version it was intended for. <BR>
 * For every application version, BOINC stores the project it belongs to
 * (#project_master_url), and obviously the application name (#app_name) and
 * its version number (#version_num). Additionally, it stores references to all
 * the files it comprises (#file_ref).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCAppVersion
{
  QString app_name;                                     ///< internal name of the application
  unsigned version_num;                                 ///< version number, without the dot (e.g. 4.05 -> 405)
  KUrl project_master_url;                              ///< project master page URL
  QList<KBSBOINCFileRef> file_ref;                      ///< references to all the files of this application version
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCAppVersion();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a specific version of an application.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/// List of application versions
typedef QList<KBSBOINCAppVersion> KBSBOINCAppVersionList;

/**
 * @brief Workunit
 *
 * Projects assign work to participating hosts in chunks called @em workunits.
 * Each workunit is assigned a unique name (#name), and is meant to be
 * processed by a specific version (#version_num) of a project application
 * client (#app_name). It's possible for the project to specify particular
 * command-line parameters to be passed to the application (#comman_line),
 * and/or environment variariables to be set (#env_vars). <BR>
 * To avoid cases in which the application malfunctions and enters an infinite
 * loop or consumes a disproportionate amount of resources (memory or disk
 * space), the project also specifies an estimate of the number of floating
 * point operations the workunit will require (#fpops_est). It will also set
 * upper bounds for these (#fpops_bound), and for (#memory_bound) and disk
 * space (#disk_bound) as well. If the computation of the workunit exceeds
 * any of these bounds, it will be aborted. <BR>
 * BOINC also associates to a workunit references to all the files (#file_ref)
 * containing the data that will serve as input for the computation, and
 * a result that will be the output of said computation.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCWorkunit
{
  QString name,                                         ///< workunit name
          app_name;                                     ///< application required to process the workunit
  unsigned version_num;                                 ///< application version we use to compute this workunit
  KUrl project_master_url;                              ///< project master page URL
  QString command_line,                                 ///< command line instruction to launch the app
          env_vars;                                     ///< environment variables
  struct {
    double fpops_est,                                   ///< estimate of floating point operations required
           fpops_bound,                                 ///< maximum floating point operations allowed
           memory_bound,                                ///< maximum memory allowed (in bytes)
           disk_bound;                                  ///< maximum disk space allowed (in bytes)
  } rsc;                                                ///< computer resources estimates and bounds
  QList<KBSBOINCFileRef> file_ref;                      ///< file reference for the workunit
  QString result_name;                                  ///< result name
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCWorkunit();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a workunit.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/// Result state
enum KBSBOINCResultState
{
  ResultNew,                                            ///< newly created workunit entry
  ResultDownloading,                                    ///< downloading workunit from server
  ResultDownloaded,                                     ///< ready to be processed, or already being processed
  ResultComputeError,                                   ///< aborted because of an error in the computation
  ResultUploading,                                      ///< done, attempting to upload the result to the server
  ResultUploaded                                        ///< done, and already uploaded the result to the server
};

/**
 * @brief Result
 *
 * A result represents the output of the computation of a workunit. As soon
 * as it starts receiving a new workunit from the project, the BOINC client
 * immediately creates an empty result. As the computation of the workunit
 * progresses, it updates the status (#state) of its result. When the
 * result has finally been computed, BOINC also records the total time
 * the computation took (#final_cpu_time) and the final exit status of
 * the application (#exit_status), and sets the #ready_to_report flag. <BR>
 * Symmetrically to what happens for a workunit, BOINC associates to a
 * result both the name of the workunit that was used as input (#wu_name), and
 * references to the files that form the output of the computation (#file_ref).
 * <BR>
 * It's possible to suspend (#suspended_via_gui) or even abort
 * (#aborted_via_gui) the computation of a result through
 * KBoincSpy or the BOINC manager application.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCResult
{
  QString name;                                         ///< result name
  double final_cpu_time;                                ///< total CPU time in seconds if done, 0 otherwise
  int exit_status;                                      ///< exit status of the application if done, 0 otherwise
  KBSBOINCResultState state;                            ///< result state @sa KBSBOINCResultState
  bool ready_to_report;                                 ///< done, and ready to be sent back to the server
  QString wu_name;                                      ///< workunit name
  KUrl project_master_url;                              ///< project master page URL
  QDateTime report_deadline;                            ///< deadline for credit to be awarded
  QList<KBSBOINCFileRef> file_ref;                      ///< file reference for the result
  bool suspended_via_gui,                               ///< execution suspended
       aborted_via_gui;                                 ///< execution aborted
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCResult();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about a result.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/// Scheduler state
enum KBSBOINCSchedulerState
{
  CPUSchedUninitialized,                                ///< the process has not started yet
  CPUSchedPreempted,                                    ///< the process has been preempted
  CPUSchedScheduled                                     ///< the process is running
};

/// Active task state
enum KBSBOINCActiveTaskState
{
  ProcessUninitialized,                                 ///< the process has not started yet
  ProcessExecuting,                                     ///< the process is running
  ProcessSuspended=9,                                   ///< the process has been suspended
  ProcessAbortPending=5,                                ///< the process has been aborted
  ProcessQuitPending=8                                  ///< the process is terminating because the boinc client is quitting
};

/**
 * @brief Active task information
 *
 * Once its computation starts, a result becomes an active task. It gets
 * assigned a unique slot number (#slot), and some space  inside the @c slots
 * subdirectory to hold any temporary file it may need. <BR>
 * In general, an active task does not necessarily stand a running program.
 * the BOINC client may choose to temporarily suspend one task and assign the
 * CPU to another. The value of @active_task_state can be inspected to
 * to determine if the task is currenlty running. <BR>
 * Computation will consist of running a workunit with a specific version
 * (#app_version_num) of an application, and saving the output into the
 * result (#result_name). At any time during the execution, the BOINC client
 * keeps track of CPU time consumed so far (#current_cpu_time), the percentage
 * of work done (#fraction_done), and the memory resources that the process
 * consumes (#vm_bytes, #rss_bytes). To avoid losing all the work in case
 * BOINC is abruptly stopped (e.g. in the case of a power outage, or a computer
 * crash), the application saves its state at regular checkpoints
 * (#checkpoint_cpu_time). <BR>
 * Some applications can open a window and show a graphical representation of
 * the work being done, when issued the @c show_graphics RPC call.
 * If the task's application has this capability, #support_graphics will be
 * set to @c true.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCActiveTask
{
  KUrl project_master_url;                              ///< project master page URL
  QString result_name;                                  ///< result name
  unsigned app_version_num,                             ///< version of the application we use to compute this workunit
           slot;                                        ///< slot number (used as unique ID for active tasks)
  KBSBOINCSchedulerState scheduler_state;               ///< scheduler state (deprecated) @sa KBSBOINCSchedulerState
  KBSBOINCActiveTaskState active_task_state;            ///< active task state @sa KBSBOINCActiveTaskState
  double checkpoint_cpu_time,                           ///< CPU time spent at the moment of last checkpoint
         fraction_done,                                 ///< percent done, expressed as a number between 0 (start) and 1 (end)
         current_cpu_time,                              ///< CPU time spent so far computing the result
         vm_bytes,                                      ///< virtual memory size (in bytes)
         rss_bytes;                                     ///< resident set size (in bytes)
  bool supports_graphics;                               ///< the application is capable of showing graphics on this platform
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCActiveTask();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains information
   * about an active task.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
  */
  bool parse(const QDomElement &node);
  
  /**
   * @brief Running task check
   *
   * Checks if the task is currently running.
   *
   * @return @c true iff the task is running
   */
  bool isRunning() const;
};

/**
 * @brief Set of active tasks
 *
 * For convenience, we index the set of active tasks both with respect to
 * slot numbers (#active_task) and result name (#slot).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCActiveTaskSet
{
  QHash<unsigned,KBSBOINCActiveTask> active_task;       ///< set of active tasks, indexed by slot number
  QHash<QString,unsigned> slot;                         ///< set of slot numbers, indexed by result name
  
  /**
   * @brief Parse function.
   *
   * Parses the subtree of the client state XML tree that contains the set of
   * active tasks.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Proxy server information
 *
 * While most computers can communicate with BOINC directly, some of those
 * which are behind firewalls may need to use a proxy server to connect to the
 * outside world. BOINC supports two types of firewalls: HTTP and SOCKS. <BR>
 * If using a proxy, the user will need to provide its DNS name or IP address
 * (#name), and the TCP/IP port it uses (#port). If the proxy requires
 * authentication, a login/password combination (#user) will also be needed.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCProxyInfo
{
  unsigned socks_version;                               ///< SOCKS major version number
  struct {
    struct {
      QString name;                                     ///< DNS name or IP address of the proxy
      unsigned port;                                    ///< proxy port number
    } server;                                           ///< server information
    struct {
      QString name,                                     ///< proxy login name, if any
              passwd;                                   ///< proxy password, if any
    } user;                                             ///< login information
  } socks, http;                                        ///< connection parameters for SOCKS and HTTP proxy servers
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCProxyInfo();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client state XML tree that contains the proxy
   * server information.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Client state
 *
 * The client state file @c client_state.xml aggregates most information about
 * the BOINC client. This includes:
 *
 *   - information about the host computer (#host_info)
 *   - statistics about computation (#time_stats) and network status
 *     (#net_stats)
 *   - the list of projects the client is attached to (#project)
 *   - the list of client applications used by these projects (#app)
 *   - for each application, one or more versions (#app_version)
 *   - a list of workunits (#workunit)
 *   - for each workunit, a corresponding result (#result)
 *   - information for files used by the project (#file_info), including (but
 *     not limited to) client applications, workunit and result files
 *   - the set of active task (#active_task_set)
 *   - proxy server parameters, if using one (#proxy_info)
 *   - version of the BOINC client (#core_client), and platform for which it
 *     was compiled
 *   - host venue (#host_venue)
 *
 * @b Note: this same information can also be obtained through the RPC call
 * @c get_state.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCClientState
{
  KBSBOINCHostInfo host_info;                           ///< host information
  KBSBOINCTimeStats time_stats;                         ///< time statistics
  KBSBOINCNetStats net_stats;                           ///< network statistics
  QHash<QString,KBSBOINCProject> project;               ///< projects, indexed by project name
  QHash<QString,KBSBOINCApp> app;                       ///< applications, indexed by application name
  QHash<QString,KBSBOINCFileInfo> file_info;            ///< files information, indexed by file name
  QHash<QString,KBSBOINCAppVersionList> app_version;    ///< application versions, indexed by application name
  QHash<QString,KBSBOINCWorkunit> workunit;             ///< workunits, indexed by workunit name
  QHash<QString,KBSBOINCResult> result;                 ///< results, indexed by result name
  KBSBOINCActiveTaskSet active_task_set;                ///< set of active tasks
  QString platform_name;                                ///< platform for which the BOINC client was built (e.g. "x86_64-pc-linux-gnu")
  struct {
    unsigned major_version,                             ///< major version number (e.g. 5.xx.xx)
             minor_version,                             ///< minor version number (e.g. x.10.xx)
             release;                                   ///< release number, if any (e.g. x.xx.45)
  } core_client;                                        ///< BOINC client version (e.g. 5.10.45)
  struct {
    unsigned period;                                    ///< deprecated
    double work_done_this_period;                       ///< deprecated
  } cpu_sched;                                          ///< deprecated
  KBSBOINCProxyInfo proxy_info;                         ///< proxy server information
  QString host_venue;                                   ///< host venue (e.g. home, work, school, etc.)
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCClientState();
  
  /**
   * @brief Parse function
   *
   * Parses the client state XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Project preferences
 *
 * The project preferences consist of the resource_share, that is duplicated
 * here from the client state file, and some project-specific preferences which
 * we currently don't collect.
 *
 * @sa KBSBOINCProject
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCProjectPreferences
{
  double resource_share;                                ///< used to compute how much CPU time to give to the project
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCProjectPreferences();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the project account XML tree that contains the
   * project preferences.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Project bookmark
 *
 * For every web bookmark, we store a user-friendly name (#name), that serves
 * as a short description, a URL to a web page (#url), and a longer description
 * of what this web link is about.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCGuiUrl
{
  QString name,                                         ///< name of the bookmark (e.g. "Message Boards")
          description;                                  ///< description of the bookmarked page
  KUrl url;                                             ///< URL of the bookmarked page
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the project account XML tree that contains a project
   * bookmark.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Collection of project bookmarks
 *
 * We store the collection of web bookmarks provided by the project in the same
 * order in which they are provided, in case such order is important.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCGuiUrls
{
  QList<KBSBOINCGuiUrl> gui_url;                        ///< list of project bookmarks
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the project account XML tree that contains the
   * collection of bookmarks suggested by the project.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Project account
 *
 * When the BOINC client attaches to a project, a corresponding account file
 * is created inside the main directory, with a name that starts with
 * "account_" (e.g. @c account_setiathome.berkeley.edu.xml ). This will collect
 * some miscellanea information and settings related to the project. Among its
 * content there will be some project-specific preferences
 * (#project_preferences), and a list of links to useful web pages connected
 * to the project (#gui_urls).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCAccount
{
  KUrl master_url;                                      ///< project master page URL
  QString authenticator,                                ///< account key
          project_name;                                 ///< user-friendly name for the project
  KBSBOINCProjectPreferences project_preferences;       ///< project preferences
  KBSBOINCGuiUrls gui_urls;                             ///< collection of project bookmarks
  
  /**
   * @brief Parse function
   *
   * Parses the project account XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true if parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Project daily statistic
 *
 * A single project daily statistics datum is a snapshot of the project
 * credits. We record the day the snapshot was taken (#day), as well as the
 * values for the total (#total_credit) and daily average (#expavg_credit)
 * credits for both host and user.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCDailyStatistics
{
  QDate day;                                            ///< day in which this snapshot was taken
  struct {
    double total_credit,                                ///< total credits
           expavg_credit;                               ///< average daily credits
  } user, host;                                         ///< user and host computer credits
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCDailyStatistics();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the project daily statistics XML tree that contains
   * a single statistics datum.
   *
   * @param node the root element of the XML subtree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Daily statistics comparison
 *
 * Compares two daily statistics with respect to the days they refer to.
 *
 * @param stat1 the first statistic
 * @param stat2 the second statistic
 * @return @c true iff the first statistic is about a day which is
 *         chronologically earlier than the one of the second
 */
KBSCORE_EXPORT bool operator<(const KBSBOINCDailyStatistics &stat1, const KBSBOINCDailyStatistics &stat2);

/**
 * @brief Project daily statistics
 *
 * A list of daily statistics for a project. Statistics are ordered by the
 * day they were taken. To identify the project we use, as usual, its master
 * page.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCProjectStatistics
{
  KUrl master_url;                                      ///< project master page URL
  QList<KBSBOINCDailyStatistics> daily_statistics;      ///< list of daily statistics, ordered by day
  
  /**
   * @brief Parse function
   *
   * Parses the project daily statistics XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Client message
 *
 * The boinc client can issues some messages to inform the user of some special
 * events or errors. Every message will have a content (#body) and timestamp
 * (#time). Since under certain circumnstances BOINC can generate
 * several messages per second, the timestamp cannot be used to reconstruct the
 * messages' correct chronological sequence. For this reason, BOINC also
 * assigns progressive sequence numbers (#seqno) to each message. Information
 * messages usually have a priority (#pri) of 0; more important messages
 * alerting the user of an error may have a higher priority number. Messages
 * may pertain a specific project (#project) the client is attached to; if
 * they don't pertain any specific project, this value will be set to
 * @c "BOINC".
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCMsg
{
  unsigned pri,                                         ///< priority
           seqno;                                       ///< sequence number
  QString body;                                         ///< message content
  QDateTime time;                                       ///< time when the message was sent
  QString project;                                      ///< project the message pertains to, if any; @c "BOINC" otherwise
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCMsg();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the client messages XML tree that contains a single
   * message.
   *
   * @param node the root element of the XML subtree
   * return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Client messages comparison
 *
 * Compares two messages with respect to their sequence numbers.
 *
 * @param msg1 the first statistic
 * @param msg2 the second statistic
 * return @c true iff the first message has a smaller sequence number than the
 *        second
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
KBSCORE_EXPORT bool operator<(const KBSBOINCMsg &msg1, const KBSBOINCMsg &msg2);

/**
 * @brief Client messages
 *
 * List of client messages, ordered by sequence number.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCMsgs
{
  QList<KBSBOINCMsg> msg;                               ///< list of messages, ordered by sequence number
  
  /**
   * @brief Parse function
   *
   * Parses the client messages XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/// Run mode
enum KBSBOINCRunMode
{
  RunAuto,                                              ///< run according to the preferences set in the BOINC web page
  RunAlways,                                            ///< run always, regardless of what the preferences say
  RunNever                                              ///< suspend execution
};

/// Network mode
enum KBSBOINCNetworkMode
{
  ConnectAlways,                                        ///< use the network when needed
  ConnectNever                                          ///< suspend all network communications
};

/**
 * @brief Persistent file transfer
 *
 * It's possible that the tranfer of a file fails for a number of reasons, like
 * the network connection being down, or the project servers being temporarily
 * off-line. When this happens, BOINC will wait a while and then re-try the
 * transfer. The persistent file transfer record contains the main data used
 * to perform this function. <BR>
 * Specifically, BOINC keeps count of the number of attempts so far
 * (#num_retries), the time of the first (@first_request_time) and next
 * scheduled one (#next_request_time). It also keeps tab of all the time
 * it has spent so far trying to transfer the file (#time_so_far).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCPersistentFileXfer
{
  unsigned num_retries;                                 ///< number of transfer attempts
  QDateTime first_request_time,                         ///< time of the first attempt
            next_request_time;                          ///< time of the next scheduled attempt
  double time_so_far;                                   ///< time elapsed so far
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCPersistentFileXfer();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the file transfers XML tree that contains the
   * persistent file transfer information for one of the files being
   * transferred.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Current file transfer
 *
 * Among the information that BOINC keeps about an ongoing file transfer
 * attempt are the number of bytes transferred so far (#bytes_xferred),
 * the average transfer speed measured (#xfer_speed), and the name of the
 * host with which the transfer is taking place. <BR>
 * For outgoing transfers, BOINC also reports the offset (from the start
 * of the file) where the current bytes being transfered are located
 * (#file_offset).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCFileXfer
{
  double bytes_xferred,                                 ///< bytes transferred so far
         file_offset,                                   ///< position of the file currently uploading
         xfer_speed;                                    ///< average transfer speed (in bytes/sec)
  QString hostname;                                     ///< server
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCFileXfer();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the file transfers XML tree that contains the
   * current transfer attempt information for one of the files being
   * transferred.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief File transfer
 *
 * The information associated to a file transfer will include the file name
 * (#name), the project it comes from or goes to (#project_url, #project_name),
 * the number of bytes already transferred (#nbytes), and the total number
 * of bytes to transfer (#max_nbytes). Since a transfer may require several
 * attempts, BOINC records information about the current attempt (#file_xfer)
 * as well as the total number of attempts so far (#persistent_file_xfer). <BR>
 * In case a transfer fails, its status will contain a negative error code
 * indicating the reason of the failure.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCFileTransfer
{
  KUrl project_url;                                     ///< project master page URL
  QString project_name,                                 ///< user-friendly project name
          name;                                         ///< file name, as stored in the filesystem
  double nbytes,                                        ///< bytes transferred so far
         max_nbytes;                                    ///< total number of bytes to transfer
  KBSBOINCPersistentFileXfer persistent_file_xfer;      ///< persistent file transfer info
  KBSBOINCFileXfer file_xfer;                           ///< current file transfer info
  int status;                                           ///< status, negative if failed
  bool generated_locally,                               ///< uploading the file (as opposed to downloading it)
       marked_for_delete;                               ///< delete the file after uploading it
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCFileTransfer();
  
  /**
   * @brief Parse function
   *
   * Parses the subtree of the file transfers XML tree that contains all the
   * information regarding the transfer of a single file.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief File transfers
 *
 * Set of file transfers, for convenience indexed by file name.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCFileTransfers
{
  QHash<QString,KBSBOINCFileTransfer> file_transfer;    ///< file transfers, indexed by file name
  
  /**
   * @brief Parse function
   *
   * Parses the file transfers XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Project account configuration
 *
 * Attaching to a BOINC project is a two-step process. In a first phase,
 * KBoincSpy contacts the project through its master page URL and gets
 * some information about how the project is configured. This information
 * determines how to carry out the second phase, which consists of
 * authenticating the user account with the server, after having created such
 * account if the user didn't have one. <BR>
 * Some old projects still use an antiquated form of authentication, that
 * requires the user to (1) registers on the project web site, (2) wait
 * for an authentication key to arrive by email, and (3) use that key
 * as authentication to attach to the project. For these projects, the
 * #client_account_creation_disabled will be set. <BR>
 * Modern BOINC projects authenticate through a more conventional
 * login/password method. Depending on the project, the login will be
 * either an email address, or a name chosen by the user (#uses_username). <BR>
 * Projects can disable registration of new accounts when they feel they
 * have as many participants as they can handle (#account_creation_disabled).
 * If account creation is not disabled, a user will be able to create an
 * account and attaching the project at the same time, without the need to
 * visit the project web page. When creating a new account, for security
 * reasons the project may specify a minimum password length policy
 * (#min_passwd_length).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCProjectConfig
{
  QString name;                                         ///< user-friendly name for the project
  bool account_manager,                                 ///< this is an Account Manager page, not a project master page
       uses_username,                                   ///< uses a user name, as opposed to a email address, as login
       account_creation_disabled,                       ///< no new users accepted
       client_account_creation_disabled;                ///< new users must register on the project web site and get an account key by email
  unsigned min_passwd_length;                           ///< minimum password length
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   */
  KBSBOINCProjectConfig();
  
  /**
   * @brief Parse function
   *
   * Parses the project configuration XML tree.
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Account manager login information
 *
 * The account manager login file stores some basic information used to
 * authenticate with the Account Manager. Aside from login (#login) and
 * password (#password_hash), it contains a copy of the host unique ID
 * (as stored in the host information section of the client state file)
 * and the time of the next scheduled contact with the AM.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCAcctMgrLogin
{
  QString login,                                        ///< login name
          password_hash,                                ///< obfuscated password
          previous_host_cpid;                           ///< host ID
  QDateTime next_rpc_time;                              ///< next scheduled contact with the Account Manager
  
  /**
   * @brief Parse function
   *
   * Parses the Account Manager login XML file (@c acct_mgr_login.xml).
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

/**
 * @brief Account manager
 *
 * At the moment the only information that KBoincSpy collects about
 * the Account Manager (if a user is attached to one) are its name (#name),
 * the URL of its main page (#url), and the login information. The first
 * two are contained in the Account Manager URL file, the last one in
 * the Account Manager Login file.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSBOINCAcctMgr
{
  QString name;                                         ///< User-friendly name
  KUrl url;                                             ///< main page URL
  KBSBOINCAcctMgrLogin login;                           ///< login information
  
  /**
   * @brief Parse function
   *
   * Parses the Account Manager URL XML file (@c acct_mgr_url.xml).
   *
   * @param node the root element of the XML tree
   * @return @c true iff parsing was successful
   */
  bool parse(const QDomElement &node);
};

#endif
