/***************************************************************************
 *   Copyright (C) 2004 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSBOINCMONITOR_H
#define KBSBOINCMONITOR_H

#include <QDomElement>
#include <QHash>
#include <QProcess>
#include <QStringList>

#include <kbsboincdata.h>
#include <kbscore_export.h>
#include <kbsdatamonitor.h>

class KBSProjectMonitor;
class KBSRPCMonitor;
class KBSTaskMonitor;
class KBSTreeNode;

struct KBSCORE_EXPORT KBSLocation {
  KUrl url;
  QString host;
  unsigned port;
  
  static QString defaultHost(const KUrl &url);
  static unsigned defaultPort;
  
  QString defaultHost() const;
};

KBSCORE_EXPORT bool operator==(const KBSLocation &location1, const KBSLocation &location2);

class KBSCORE_EXPORT KBSBOINCMonitor : public KBSDataMonitor
{
  Q_OBJECT
  public:
    KBSBOINCMonitor(const KBSLocation &location, KBSTreeNode *parent=0);
    virtual ~KBSBOINCMonitor();
    
    virtual bool isLocal() const;
    virtual KBSLocation location() const;
    
    virtual void exec(const KUrl &client, bool killOnExit=true);
    
    virtual const KBSBOINCClientState *state() const;
    virtual const KBSBOINCAccount *account(const QString &project) const;
    virtual const KBSBOINCProjectStatistics *statistics(const QString &project) const;
    virtual const KBSBOINCAcctMgr *acctMgr() const;
    
    virtual QString project(const KBSBOINCProject &project) const;
    virtual QString project(const KBSBOINCApp &app) const;
    virtual QString project(const KBSBOINCWorkunit &workunit) const;
    virtual QString project(const KBSBOINCResult &result) const;
    virtual QString project(const KBSBOINCActiveTask &task) const;
    virtual QString project(const KBSBOINCAccount &account) const;
    virtual QString project(const KBSBOINCProjectStatistics &statistics) const;
    
    virtual QString app(const KBSBOINCApp &app) const;
    virtual QString app(const KBSBOINCWorkunit &workunit) const;
    virtual QString app(const KBSBOINCResult &result) const;
    virtual QString app(const KBSBOINCActiveTask &task) const;
    
    virtual QString workunit(const KBSBOINCWorkunit &workunit) const;
    virtual QString workunit(const KBSBOINCResult &result) const;
    virtual QString workunit(const KBSBOINCActiveTask &task) const;
    
    virtual QString result(const KBSBOINCResult &result) const;
    virtual QString result(const KBSBOINCActiveTask &task) const;
  
  public:
    virtual KBSRPCMonitor *rpcMonitor() const;
    virtual KBSProjectMonitor *projectMonitor(const QString &project) const;
    virtual KBSTaskMonitor *taskMonitor(unsigned task) const;
    
    friend class KBSProjectMonitor;
    
  signals:
    void projectsAdded(const QStringList &projects);
    void projectsRemoved(const QStringList &projects);
    void appsAdded(const QStringList &apps);
    void appsRemoved(const QStringList &apps);
    void workunitsAdded(const QStringList &workunits);
    void workunitsRemoved(const QStringList &workunits);
    void resultsAdded(const QStringList &results);
    void resultsRemoved(const QStringList &results);
    void resultsCompleted(const QStringList &results);
     
    void workunitActivated(unsigned task, const QString &workunit, bool activated);
    void resultActivated(unsigned task, const QString &result, bool activated);
    
    void stateUpdated();
    void accountUpdated(const QString &project);
    void statisticsUpdated(const QString &project);
    void acctMgrUpdated();
  
  protected:
    virtual bool parseFile(KBSFileInfo *file, const QString &fileName);
    
    static QString formatAccountFileName(const QString &project);
    static QString parseAccountFileName(const QString &fileName);
    
    static QString formatStatisticsFileName(const QString &project);
    static QString parseStatisticsFileName(const QString &fileName);
  
  private:
    bool parseClientStateDocument(const QDomDocument &document);
    bool parseAccountDocument(const QDomDocument &document, KBSBOINCAccount &account);
    bool parseStatisticsDocument(const QDomDocument &document, KBSBOINCProjectStatistics &statistics);
    bool parseAcctMgrDocument(const QDomDocument &document);
    bool parseAcctMgrLoginDocument(const QDomDocument &document);
    
    void notify(const QString &message, const QString &text);
  
  private slots:
    void addProjectFiles(const QStringList &projects);
    void removeProjectFiles(const QStringList &projects);
    
    void addProjectMonitors(const QStringList &projects);
    void removeProjectMonitors(const QStringList &projects);
    
    void updateTaskMonitor(unsigned task, const QString &result, bool add);
    
    void updateFile(const QString &fileName);
  
  protected:
    KBSBOINCClientState m_state;
    QHash<QString,KBSBOINCAccount*> m_accounts;
    QHash<QString,KBSBOINCProjectStatistics*> m_statistics;
    KBSBOINCAcctMgr m_acctMgr;
  
  protected:
    KBSLocation m_location;
    KBSRPCMonitor *m_rpcMonitor;
    QHash<QString,KBSProjectMonitor*> m_projectMonitors;
    QHash<unsigned,KBSTaskMonitor*> m_taskMonitors;
    QProcess *m_client;
  
  private:
    bool m_startup, m_exec;
    QStringList m_projects, m_apps, m_workunits, m_results, m_complete;
    QHash<unsigned,KBSBOINCActiveTask> m_tasks;
    QList<unsigned> m_running;
};

#endif
