/***************************************************************************
 *   Copyright (C) 2004-2008 by Roberto Virga                              *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSDATAMONITOR_H
#define KBSDATAMONITOR_H

#include <QDateTime>
#include <QDomDocument>
#include <QHash>
#include <QStringList>
#include <QTimerEvent>

#include <KDE/KDirWatch>
#include <KDE/KIO/Job>
#include <KDE/KTemporaryFile>
#include <KDE/KUrl>

#include <kbscore_export.h>

/**
 * @brief File information
 *
 * For every files it monitors, KBoincSpy keeps some basic information about
 * its status, such as the name (#filename), whether it exists (#exists), and,
 * if it does, the date of its last change (#timestamp) and its size (#size).
 * Since we'll be interested in parsing its contents, in full or at least
 * partially, we'll also be interested if such parsing completed sucessfully
 * (#ok). <BR>
 * For some files, it is known in advance that, once created, they will not
 * change their contents. It is therefore pointless and expensive to poll them
 * periodically for changes. For these files, we will clear the #monitored
 * flag, and KBoincSpy will poll them only until they get parsed successfully
 * once. When that happens, it will set the #initialized flag, and it will
 * never look at them again.
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
struct KBSCORE_EXPORT KBSFileInfo
{
  QString fileName;             ///< file name
  bool initialized,             ///< has it been parsed successfully at least once?
       monitored,               ///< is it still being monitored?
       exists;                  ///< does it currently exist?
  QDateTime timestamp;          ///< date and time of last change
  qint64 size;                  ///< current size
  bool ok;                      ///< has it been parsed successfully?
  
  /**
   * @brief Constructor
   *
   * Assigns sensible defaults to numeric and boolean fields.
   *
   * @param fileName file name
   *
   */
  KBSFileInfo(const QString &fileName);
};

/**
 * @brief File monitor
 *
 * This is the main class used by KBoincSpy for file monitoring. All files are
 * assumed to reside in a same directory, either local or remotely accessible
 * through one of KDE's ioslaves. The URL of such directory is provided during
 * initialization. <BR>
 * Files to monitor are added by calling #addFile, and removed by #removeFile.
 * We monitor the files by polling them periodically. The interval in seconds
 * between polls can be set using #setInterval; setting a zero or negative
 * suspends all polling activity. Besides this periodic polling, that happens
 * automatically, you can force a manual one by calling #checkFiles. <BR>
 * There are some files, typically input ones, that never change once created.
 * To avoid pointless and possibly expensive polling (if they are stored on a
 * remote host), you can use #setMonitoring to set their monitoring status to
 * @c false. If you do that, the files will be polled only until successfully
 * parsed. <BR>
 * If a monitored file has been created or changed, KBoincSpy will get a local
 * copy of it (this will be the file itself if it was a local file) and call
 * #parseFile. You can (and should) reimplement this method to parse the file
 * and return @c true iff such parsing was successful. Feel free to use the
 * #readFile and #readDevice for reading the file into memory and converting
 * it to a more convenient format (list of lines, XML DOM tree, etc.).
 *
 * @author Roberto Virga <rvirga@users.sourceforge.net>
 */
class KBSCORE_EXPORT KBSDataMonitor : public QObject
{
  Q_OBJECT
  Q_PROPERTY(KUrl url READ url)
  Q_PROPERTY(int interval READ interval
                          WRITE setInterval
                          NOTIFY intervalChanged)
  public:
    /**
     * @brief Destructor
     *
     * Destroy the file monitor.
     *
     */
    virtual ~KBSDataMonitor();
    
    /**
     * @brief Base URL
     *
     * URL of the (local or remote) directory containing the monitored files.
     *
     * @return the base URL
     *
     */
    virtual KUrl url() const;
    
    /**
     * @brief Poll interval
     *
     * Frequency (in seconds) at which the monitored files are being polled.
     *
     * @return the poll interval (in seconds)
     *
     * @sa setInterval(int interval)
     *
     */
    virtual int interval() const;
  
  public slots:
    /**
     * @brief Set poll interval
     *
     * Set the frequency (in seconds) at which the monitored files are being
     * polled. If called with a nonpositive number, polling is suspended.
     *
     * @param interval poll interval (in seconds)
     *
     * @sa interval()
     *
     */
    virtual void setInterval(int interval);
    /**
     * @brief Manual polling
     *
     * Force a manual polling of all monitored files.
     *
     */
    virtual void checkFiles();
  
  signals:
    /**
     * @brief Poll interval changed
     *
     * Emitted when we change the polling interval.
     *
     * @param interval new poll interval (in seconds)
     *
     * @sa setInterval(int interval)
     *
     */
    void intervalChanged(int interval);
    /**
     * @brief File updated
     *
     * Emitted when a monitored file is created, destroyed, or changed.
     * @note If the file has been created or changed, this signal is emitted
     * @em after the file has been parsed.
     *
     * @param fileName monitored file name
     *
     */
    void fileUpdated(const QString &fileName);
  
  protected:
    /**
     * @brief Constructor
     *
     * Construct a new file monitor for the given directory.
     *
     * @param url URL of the directory containing the files we want to monitor
     * @param parent parent object
     *
     */
    KBSDataMonitor(const KUrl &url, QObject *parent=0);
    
    /**
     *
     * @brief Timer event
     *
     * Reimplemented to catch the polling timer events.
     *
     * @param e timer event record
     *
     */
    virtual void timerEvent(QTimerEvent *e);
    
    /**
     * @brief Start monitoring a file
     *
     * Adds the given name to the list of monitored files.
     *
     * @param fileName file name
     *
     */
    virtual void addFile(const QString &fileName);
    /**
     * @brief Stop monitoring a file
     *
     * Removes the given name from the list of monitored files.
     *
     * @param fileName file name
     *
     */
    virtual void removeFile(const QString &fileName);
    /**
     * @brief Change monitoring status
     *
     * By default, a file is monitored for change during its entire lifespan.
     * If you know a file isn't going to change, you can call this method with
     * @c monitored equal to @c false to stop monitoring the file once it has
     * been correctly parsed a first time.
     *
     * @param fileName file name
     * @param monitored if @c true, we stop polling it once we've parsed it
     *
     */
    virtual void setMonitoring(const QString &fileName, bool monitored);
    
    /**
     * @brief File status
     *
     * Retrieve the current status of the given (monitored) file.
     *
     * @param fileName file name
     * @return the file's status
     *
     */
    virtual const KBSFileInfo *file(const QString &fileName) const;
    
    /**
     * @brief Utility function for reading a file
     *
     * Static convenience functions that reads a file and converts the content
     * into an XML DOM document.
     *
     * @param fileName file name
     * @param content file content as an XML DOM document
     * @return @c true iff no errors have occurred
     *
     */
    static bool readFile(const QString &fileName, QDomDocument &content);
    /**
     * @brief Utility function for reading a file
     *
     * Static convenience function that reads a file into a string. Optionally,
     * you can specify the position from which you want to start reading.
     *
     * @param fileName file name
     * @param content file content as a string
     * @param start file offset from which we start reading
     * @return @c true iff no errors have occurred
     *
     */
    static bool readFile(const QString &fileName, QString &content, qint64 start=0);
    /**
     * @brief Utility function for reading a file
     *
     * Static convenience function that reads a file into a sequence of lines.
     * Optionally, you can specify the position from which you want to start
     * reading, and a delimiter at which to stop.
     *
     * @param fileName file name
     * @param content file content as a sequence of lines
     * @param start file offset from which we start reading
     * @param delimiter delimiter at which we stop reading
     * @return @c true iff no errors have occurred
     *
     */
    static bool readFile(const QString &fileName, QStringList &content,
                         qint64 start=0, const QString &delimiter=QString::null);
    
    /**
     * @brief Utility function for reading a stream
     *
     * Static convenience function that reads a stream and converts it into an
     * XML DOM document.
     *
     * @param device IO stream
     * @param content stream content as an XML DOM document
     * @return @c true iff no errors have occurred
     *
     */
    static bool readDevice(QIODevice *device, QDomDocument &content);
    /**
     * @brief Utility function for reading a stream
     *
     * Static convenience function that reads a stream into a string.
     *
     * @param device IO stream
     * @param content stream content as a string
     * @return @c true iff no errors have occurred
     *
     */
    static bool readDevice(QIODevice *device, QString &content);
    /**
     * @brief Utility method for reading a stream
     *
     * Static convenience function that reads a stream into a sequence of
     * lines. Optionally, you can specify a delimiter at which you want to stop
     * reading.
     *
     * @param device IO stream
     * @param content file content as a sequence of lines
     * @param delimiter delimiter at which we stop reading
     * @return @c true iff no errors have occurred
     *
     */
    static bool readDevice(QIODevice *device, QStringList &content,
                           const QString &delimiter=QString::null);
    
    /**
     * @brief File parsing
     *
     * Parse a monitored file. For local files, #fileName is the full path of
     * the file. For remote files, #fileName is the path to a temporary file
     * where we have previously copied the remote file's contents.
     *
     * @param file file information about the monitored file
     * @param path path to a local file holding the contents of the monitored
     *             file
     * @return @c true iff parsing was successful
     *
     */
    virtual bool parseFile(KBSFileInfo *file, const QString &path);
  
  private:
    /**
     * @brief File polling
     *
     * Poll the monitored file whose info are passed as parameter, and updates
     * said parameter. If it is a local file, it uses #updateLocalFileInfo,
     * otherwise it queues a new stat job using #queueStatJob.
     *
     * @param info information about the file to be polled
     *
     */
    void checkFile(KBSFileInfo *info);
    
    /**
     * @brief Update local file information
     *
     * Poll the file at the given path, and update the file information record.
     *
     * @param path path to the file to be polled
     * @param info information record to be updated
     *
     */
    void updateLocalFileInfo(const QString &path, KBSFileInfo *info);
    
    /**
     * @brief Queue a stat job
     *
     * Queue a stat (poll file information) job.
     *
     * @param fileName name of the file to be polled
     *
     */
    void queueStatJob(const QString &fileName);
    /**
     * @brief Queue a copy job
     *
     * Queue a copy job. Once we determine a remote file has been modified,
     * we need to copy its content to a (temporary) local one in order to be
     * parsed.
     *
     * @param fileName name of the file to be copied
     *
     */
    void queueCopyJob(const QString &fileName);
    
    /**
     * @brief Start a stat job
     *
     * Start a stat (poll file information) job.
     *
     * @param fileName name of the file to be polled
     *
     */
    void commenceStatJob(const QString &fileName);
    /**
     * @brief Start a copy job
     *
     * Start a copy job. Once we determine a remote file has been modified,
     * we need to copy its content to a (temporary) local one in order to be
     * parsed.
     *
     * @param fileName name of the file to be copied
     *
     */
    void commenceCopyJob(const QString &fileName);
  
  private slots:
    /**
     * @brief File polling
     *
     * Poll the given path, if it corresponds to a monitored file.
     * Called by KDirWatch when a file in the base directory is
     * created/destroyed/modified.
     *
     * @param path path to the file being polled
     *
     */
    void checkFile(const QString &path);
    
    /**
     * @brief Stat job completed
     *
     * Called by KIO when a stat job (previously started by #commenceStatJob)
     * has been completed.
     *
     * @param job stat job info and outcome
     *
     */
    void statResult(KIO::Job *job);
    /**
     * @brief Copy job completed
     *
     * Called by KIO when a copy job (previously started by #commenceCopyJob)
     * has been completed.
     *
     * @param job copy job info and outcome
     *
     */
    void copyResult(KIO::Job *job);
  
  private:
    KUrl m_url;                          ///< base URL
    int m_interval,                      ///< poll interval
        m_timer;                         ///< ID of the timer used for polling
    QHash<QString,KBSFileInfo*> m_files; ///< file status hash
    KDirWatch *m_watch;                  ///< for local files, we use inotify in addition to polling
    KIO::Job *m_job;                     ///< used for stat and copy jobs
    KTemporaryFile *m_tmp;               ///< temporary local file used for copy jobs
    QStringList m_stat,                  ///< stat jobs queue
                m_copy;                  ///< copy jobs queue
};

#endif
