/***************************************************************************
 *   Copyright (C) 2008 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSFIELD_H
#define KBSFIELD_H

#include <QColor>
#include <QString>

#include <KDE/KUrl>

#include <kbscore_export.h>

/**
 * @brief Field
 *
 * A field is a pair key/value visualized inside a panel. KBoincSpy uses a
 * model/view approach to represent fields. #KBSField is the model class.
 * A field can be either be empty, have a text value, or have a URL value.
 * Empty fields should be displayed as totally blank. For text-valued fields,
 * we distinguish between the cases when we can bound a priori the length of
 * the text and when we cannot. Finally, if the value is a URL, we allow
 * some auxiliary text to accompany the URL; this text should be displayed
 * as trailing the URL. <BR>
 * It's possible to attach a tooltip to a field. For URL-valued fields, this
 * is very useful, as a tooltip can be used to inform the user about where
 * the link leads to. It's also possible to attach a tooltip to a text field,
 * to provide some additional information about the text value.
 */
class KBSCORE_EXPORT KBSField : public QObject
{
  Q_OBJECT
  Q_ENUMS(Type)
  Q_PROPERTY(Type type READ type WRITE setType)
  Q_PROPERTY(QString key READ key WRITE setKey)
  Q_PROPERTY(QString text READ text WRITE setText)
  Q_PROPERTY(KUrl url READ url WRITE setUrl)
  Q_PROPERTY(QString aux READ aux WRITE setAux)
  Q_PROPERTY(QString toolTip READ toolTip WRITE setToolTip)
  Q_PROPERTY(QColor color READ color WRITE setColor)
  public:
    /// Field types
    enum Type {
      Empty,            ///< empty field
      FixedText,        ///< text field with a known maximum size
      VariableText,     ///< text field with an unknown maximum size
      Url               ///< URL field
    };
    
    /**
     * @brief Constructor
     *
     * Construct a new empty field.
     *
     * @param parent parent object
     *
     */
    KBSField(QObject *parent=0);
    
    /**
     * @brief Type
     *
     * The type of a field.
     *
     * @return the type of a field.
     *
     * @sa setType(Type type)
     *
     */
    Type type() const;
    /**
     * @brief Change the type
     *
     * Change the type of a field. If the type is changed, a #typeChanged
     * signal will be triggered. Also, the value will be cleared, and, if the
     * new type is @c Empty, the key and the tooltip as well.
     *
     * @sa type()
     *
     */
    void setType(Type type);
    
    /**
     * @brief Key
     *
     * Key part of the key/value pair. For empty fields, this is the null
     * string.
     *
     * @return the key of the field
     *
     * @sa setKey(const QString &key)
     *
     */
    QString key() const;
    /**
     * @brief Change the key
     *
     * Change the key of a field. A #keyChanged signal will be triggered.
     * Note that changing the key of an empty field will have no effects.
     *
     * @param key the new key
     *
     * @sa key()
     *
     */
    void setKey(const QString &key);
    
    /**
     * @brief Text component of the value
     *
     * Text component of the value of a field. For text fields (either fixed-
     * or variable-sized), this will be the entire value. For URL fields,
     * it will be the text of the URL. Finally, for empty fields this
     * function will return the null string.
     *
     * @return the text component of the value
     *
     * @sa setText(const QString &text)
     *
     */
    QString text() const;
    /**
     * @brief Change the text component of the value
     *
     * Change the text component of the value of a field. Called without
     * arguments, or with a null string, sets the value to the default
     * "unknown" value. A #textChanged signal will be triggered. Note that
     * changing the text component of an empty field will have no effects.
     *
     * @param text the new text
     *
     * @sa text()
     *
     */
    void setText(const QString &text=QString::null);
    
    /**
     * @brief URL component of the value
     *
     * URL component of the value of a URL field. For non-URL fields,
     * this function will return an empty (invalid) URL.
     *
     * @return the URL component of the value
     *
     * @sa setURL(const KUrl &url)
     *
     */
    KUrl url() const;
    /**
     * @brief Change the URL component of the value
     *
     * Change the URL component of the value of a URL field. A #urlChanged
     * signal will be triggered. Note that changing the URL component of a
     * non-URL field will have no effects.
     *
     * @param url the new URL
     *
     * @sa url()
     *
     */
    void setUrl(const KUrl &url);
    
    /**
     * @brief Auxiliary text component of the value
     *
     * Auxiliary text component of the value of a URL field. For non-URL
     * fields, this function will return a null string.
     *
     * @return the auxiliary text component of the value
     *
     * @sa setAux(const QString &aux)
     *
     */
    QString aux() const;
    /**
     * @brief Change the auxiliary text component of the value
     *
     * Change the auxiliary text component of the value of a URL field. A
     * #auxChanged signal will be triggered. Note that changing the auxiliary
     * text component of a non-URL field will have no effects.
     *
     * @param aux the new auxiliary text
     *
     * @sa aux()
     *
     */
    void setAux(const QString &aux);
    
    /**
     * @brief tooltip
     *
     * Tooltip text of a field. The tooltip of an empty field is always the
     * null string.
     *
     * @return the field tooltip
     *
     * @sa setToolTip(const QString &toolTip)
     *
     */
    QString toolTip() const;
    /**
     * @brief Change the tooltip
     *
     * Change the tooltip of a field. A #toolTipChanged signal will be
     * triggered. You can remove a tooltip by passing the null string. Note
     * that changing the tooltip of an empty field will have no effects.
     *
     * @param toolTip the new tooltip text
     *
     * @sa toolTip()
     *
     */
    void setToolTip(const QString &toolTip=QString::null);
    
    /**
     * @brief color
     *
     * Tooltip text of a field. The tooltip of an empty field is always the
     * null string.
     *
     * @return the field tooltip
     *
     * @sa setColor(const QColor &color)
     *
     */
    QColor color() const;
    /**
     * @brief Change the color
     *
     * Change the color of a field. A #colorChanged signal will be triggered.
     * You can restore the default color by passing an invalid color. In the
     * case of a URL field, only the auxiliary text field will be affected.
     * Note that changing the color of an empty field will have no effects.
     *
     * @param color the new color
     *
     * @sa color()
     *
     */
    void setColor(const QColor &color=QColor());
  
  signals:
    /**
     * @brief Type changed
     *
     * Emitted when the type has changed.
     *
     * @sa setType(Type type)
     *
     */
    void typeChanged();
    /**
     * @brief Key changed
     *
     * Emitted when the key has changed.
     *
     * @param key the new key
     *
     * @sa setKey(const QString &key)
     *
     */
    void keyChanged(const QString &key);
    /**
     * @brief Text component of the value changed
     *
     * Emitted when the text component of the value in a nonempty field has
     * changed.
     *
     * @param text the new text
     *
     * @sa setText(const QString &text)
     *
     */
    void textChanged(const QString &text);
    /**
     * @brief URL component changed
     *
     * Emitted when the URL component of the value in a URL field has changed.
     *
     * @param url the new URL
     *
     * @sa setURL(const KUrl &url)
     *
     */
    void urlChanged(const KUrl &url);
    /**
     * @brief Auxiliary text component changed
     *
     * Emitted when the auxiliary text component of the value in a URL field
     * has changed.
     *
     * @param aux the new auxiliary text
     *
     * @sa setAux(const QString &aux)
     *
     */
    void auxChanged(const QString &aux);
    /**
     * @brief Tooltip changed
     *
     * Emitted when the tooltip of a nonempty field has changed.
     *
     * @param toolTip the new tooltip text
     *
     * @sa setToolTip(const QString &toolTip)
     *
     */
    void toolTipChanged(const QString &toolTip);
    /**
     * @brief Color changed
     *
     * Emitted when the color of a nonempty field has changed.
     *
     * @param toolTip the new tooltip text
     *
     * @sa setToolTip(const QString &toolTip)
     *
     */
    void colorChanged(const QColor &color);
  
  private:
    Type m_type;        ///< type
    QString m_key,      ///< key
            m_text,     ///< text component of the value (nonempty fields only)
            m_aux,      ///< auxiliary text component (URL fields only)
            m_toolTip;  ///< tooltip (nonempty fields only)
    KUrl m_url;         ///< URL component of the value (URL fields only)
    QColor m_color;     ///< color (nonempty fields only)
};

#endif
