/***************************************************************************
 *   Copyright (C) 2008 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSFIELDBAR_H
#define KBSFIELDBAR_H

#include <QProgressBar>

#include <kbscore_export.h>

class KBSField;

/**
 * @brief Progress bar view of a field
 *
 * This widget offers a progress bar view of the text component of the value of
 * a (nonempty) field. It assumes said text component to be a floating point
 * number, optionally followed by a percent ('%') sign. This number gets
 * truncated to an integer and serves as value of the progress bar.
 *
 */
class KBSCORE_EXPORT KBSFieldBar : public QProgressBar
{
  Q_OBJECT
  public:
    /**
     *
     * @brief Constructor
     *
     * Construct a new bar view for a field. The underlying field is not set
     * here (to allow this widget to be used in GUI design tools like Qt
     * Designer), and instead will have to be set later using #setField.
     *
     * @param parent the parent widget (if any)
     *
     */
    KBSFieldBar(QWidget *parent=0);
    
    /** @brief Field
     * 
     * Return the underlying field this class serves a view of.
     *
     * @return the underlying field
     *
     * @sa setField(KBSField *field)
     *
     */
    virtual KBSField *field() const;
    /**
     *
     * @brief Change field
     *
     * Set, or change (if we already set it before) the field this class serves
     * as view for.
     *
     * @sa field()
     *
     */
    virtual void setField(KBSField *field);
  
  private slots:
    /**
     *
     * @brief Change text component of value
     *
     * Catch a change in the text component of the value of the underlying
     * field and update the value of the progress bar accordingly.
     *
     * @param text the new text
     *
     */
    void changeText(const QString &text);
    /**
     *
     * @brief Change tooltip
     *
     * Catch a change of tooltip of the undelying field and update the tooltip
     * accordingly.
     *
     * @param toolTip the new tooltip text
     *
     */
    void changeToolTip(const QString &toolTip);
  
  private:
    KBSField *m_field;  ///< underlying field (model) of this class
};

#endif
