/***************************************************************************
 *   Copyright (C) 2004-2008 by Roberto Virga                              *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSFIELDTEXT_H
#define KBSFIELDTEXT_H

#include <QLabel>
#include <QLayout>

#include <KDE/KUrlLabel>
#include <KDE/KSqueezedTextLabel>

#include <kbscore_export.h>

class KBSField;

/**
 * @brief Text of a field
 *
 * This widget offers the textual, canonical view of a field.
 *
 */
class KBSCORE_EXPORT KBSFieldText : public QWidget
{
  Q_OBJECT
  public:
    /**
     *
     * @brief Constructor
     *
     * Construct a new canonical view for a field. The underlying field is not
     * set here (to allow this widget to be used in GUI design tools like Qt
     * Designer), and instead will have to be set later using #setField.
     *
     * @param parent parent widget
     *
     */
    KBSFieldText(QWidget *parent=0);
    
    /** @brief Field
     * 
     * Return the underlying field this class serves a view of.
     *
     * @return the underlying field
     *
     * @sa setField(KBSField *field)
     *
     */
    virtual KBSField *field() const;
    /**
     *
     * @brief Change field
     *
     * Set, or change (if we already set it before) the field this class serves
     * as view for.
     *
     * @sa field()
     *
     */
    virtual void setField(KBSField *field);
  
  private slots:
    /**
     *
     * @brief Change field type
     *
     * Catch a change of type of the underlying field and update the widget
     * accordingly.
     *
     */
    void changeType();
    /**
     *
     * @brief Change key
     *
     * Catch a change in the key of the underlying field and update the widget
     * accordingly.
     *
     * @param key the new key
     *
     */
    void changeKey(const QString &key);
    /**
     *
     * @brief Change text component of value
     *
     * Catch a change in the text component of the value of the underlying
     * field and update the widget accordingly.
     *
     * @param text the new text
     *
     */
    void changeText(const QString &text);
    /**
     *
     * @brief Change URL component of value
     *
     * Catch a change in the URL component of the value of the underlying
     * URL field and update the widget accordingly.
     *
     * @param url the new URL
     *
     */
    void changeUrl(const KUrl &url);
    /**
     *
     * @brief Change auxiliary text of URL value
     *
     * Catch a change in the auxiliary text of the value of the underlying
     * URL field and update the widget accordingly.
     *
     * @param aux the new auxiliary text
     *
     */
    void changeAux(const QString &aux);
    /**
     *
     * @brief Change tooltip
     *
     * Catch a change of tooltip of the undelying field and update the tooltip
     * accordingly.
     *
     * @param toolTip the new tooltip text
     *
     */
    void changeToolTip(const QString &toolTip);
    /**
     *
     * @brief Change color
     *
     * Catch a change of color of the undelying field and update the color
     * accordingly.
     *
     * @param color the new tooltip text
     *
     */
    void changeColor(const QColor &color);
    
    /**
     *
     * @brief Launch URL
     *
     * Called when the user clicks on a link, it launches the default browser
     * with the selected URL.
     *
     * @param url the selected URL
     *
     */
    void handleUrl(const QString &url);
  
  private:
    KBSField *m_field;                  ///< underlying field (model) of this class
    
    QBoxLayout *m_layout;               ///< the widget has the layout of a horizontal box
    QLabel *m_key,                      ///< child widget containing the key
           *m_fixed,                    ///< child widget containing fixed-text values
           *m_aux;                      ///< child widget containing the auxiliary text
    KSqueezedTextLabel *m_variable;     ///< child widget containing variable-text values
    KUrlLabel *m_url;                   ///< child widget containing a clickable URL value
};

#endif
