/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSKBSMOLBIODATA_H
#define KBSKBSMOLBIODATA_H

#include <QDateTime>
#include <QStringList>

#include <kbsmolbio_export.h>

enum KBSAminoAcid
  {GLY, ALA, SER, CYS, VAL, THR, ILE, PRO, MET, ASP,
   ASN, LEU, LYS, GLU, GLN, ARG, HIS, PHE, TYR, TRP,
   AminoAcids};

enum KBSFeature {Helix, Strand, Coil, Features};

enum KBSElement
  {Hydrogen,
   Helium,
   Lithium,
   Boron,
   Carbon,
   Nitrogen,
   Oxygen,
   Fluorine,
   Neon,
   Sodium,
   Magnesium,
   Aluminium,
   Silicon,
   Phosphorus,
   Sulphur,
   Chlorine,
   Argon,
   Potassium,
   Calcium,
   Titanium,
   Chromium,
   Manganese,
   Iron,
   Nickel,
   Copper,
   Zinc,
   Gallium,
   Arsenic,
   Selenium,
   Bromine,
   Krypton,
   Palladium,
   Silver,
   Cadmium,
   Indium,
   Tin,
   Tellurium,
   Iodine,
   Xenon,
   Barium,
   Platinum,
   Gold,
   Mercury,
   Thallium,
   Lead,
   Uranium,
   Elements};

namespace KBSMolBio
{
  KBSMOLBIO_EXPORT double mass(KBSAminoAcid aa);
  KBSMOLBIO_EXPORT unsigned atoms(KBSAminoAcid aa);
  
  KBSMOLBIO_EXPORT KBSAminoAcid parseAminoAcid(QChar abbrev);
  KBSMOLBIO_EXPORT KBSAminoAcid parseAminoAcid(const QString &name);
  KBSMOLBIO_EXPORT QString formatAminoAcid(KBSAminoAcid aa);
  
  KBSMOLBIO_EXPORT KBSFeature parseFeature(QChar abbrev);
  
  KBSMOLBIO_EXPORT double atomRadius(KBSElement element);
  
  KBSMOLBIO_EXPORT KBSElement parseElement(const QString &string);
  KBSMOLBIO_EXPORT QString formatElement(KBSElement element);
};

struct KBSMOLBIO_EXPORT KBSProteinAtom
{
  unsigned x, y, z;
  KBSFeature feature;
  
  bool parse(const QString &line);
};

struct KBSMOLBIO_EXPORT KBSProteinChain
{
  QList<KBSProteinAtom> atom;
  
  bool hasStructure() const;
  void predictStructure();
  
  bool parse(const QStringList &lines);
};

struct KBSMOLBIO_EXPORT KBSHeaderPDB
{
  QString classification;
  QDate depDate;
  QString idCode;
  
  bool parse(const QString &line);
};

struct KBSMOLBIO_EXPORT KBSAtomPDB
{
  unsigned serial;                    // Atom serial number
  struct {
    unsigned remoteness,              // Distance from the alpha carbon
             branch;                  // Branch
    QString iupac;                    // Atom name in IUPAC format
  } name;
  QChar altLoc;                       // Alternate location indicator
  KBSAminoAcid resName;               // Residue name
  QChar chainID;                      // Chain identifier
  unsigned resSeq;                    // Residue sequence number
  QChar iCode;                        // Code for insertion of residues
  double x,                           // Orthogonal coordinates for X in Angstroms
         y,                           // Orthogonal coordinates for Y in Angstroms
         z,                           // Orthogonal coordinates for Z in Angstroms
         occupancy,                   // Occupancy
         tempFactor;                  // Temperature factor
  QString segID;                      // Segment identifier
  KBSElement element;                 // Element symbol
  QString charge;                     // Charge on the atom
  
  bool covalentBond(const KBSAtomPDB &atom) const;
  
  bool parse(const QString &line);
};

enum KBSHelixClass
{ RightHandedApha=1, RightHandedOmega, RightHandedPi, RightHandedGamma, RightHanded310,
  LeftHandedAlpha, LeftHandedOmega, LeftHandedGamma, Ribbon27, Polyproline };

struct KBSMOLBIO_EXPORT KBSHelixPDB
{
  unsigned serNum;                    // Helix serial number
  QString helixID;                    // Helix identifier
  struct {
    KBSAminoAcid resName;             // Name of the residue
    QChar chainID;                    // Chain identifier
    unsigned seqNum;                  // Sequence number
    QChar iCode;                      // Insertion code
  } init, end;
  KBSHelixClass helixClass;           // Helix class
  QString comment;                    // Comment about the helix
  unsigned length;                    // Length of the helix
  
  bool parse(const QString &line);
};

struct KBSMOLBIO_EXPORT KBSSheetPDB
{
  unsigned strand;                    // Strand number
  QString sheetID;                    // Sheet identifier
  unsigned numStrands;                // Number of strands in sheet
  struct {
    KBSAminoAcid resName;             // Residue name
    QChar chainID;                    // Chain identifier
    unsigned seqNum;                  // Residue sequence number
    QChar iCode;                      // Insertion code
  } init, end;
  int sense;                          // Sense (0=first, 1=parallel, -1=anti-parallel)
  struct {
    QString atom;                     // Atom
    KBSAminoAcid resName;             // Residue name
    QChar chainId;                    // Chain identifier
    unsigned resSeq;                  // Residue sequence number
    QChar iCode;                      // Insertion code
  } curr, prev;                       // Registration
  
  bool parse(const QString &line);
};

struct KBSMOLBIO_EXPORT KBSTurnPDB
{
  unsigned seq;                       // Turn number
  QString turnID;                     // Turn identifier
  struct {
    KBSAminoAcid resName;             // Name of the residue
    QChar chainID;                    // Chain identifier
    unsigned seqNum;                  // Sequence number
    QChar iCode;                      // Insertion code
  } init, end;
  QString comment;                    // Comment about the turn
  
  bool parse(const QString &line);
};

struct KBSMOLBIO_EXPORT KBSProteinPDB
{
  KBSHeaderPDB header;
  QString title;
  
  unsigned maxSeq;
  QList<unsigned> group;
  QList<KBSAtomPDB> atom;
  QList<KBSHelixPDB> helix;
  QList<KBSSheetPDB> sheet;
  QList<KBSTurnPDB> turn;
  
  unsigned groups() const;
  bool hasStructure() const;
  void predictStructure(double threshold=-0.5);
  
  bool parse(const QStringList &lines);
  QString toString() const;
};

struct KBSMOLBIO_EXPORT KBSProteinFASTA
{
  QString header;
  QList<KBSAminoAcid> sequence;
  
  bool parse(const QStringList &lines);
  QString toString() const;
  unsigned atoms() const;
};

#endif

