/***************************************************************************
 *   Copyright (C) 2006 by Roberto Virga                                   *
 *   rvirga@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Library General Public License as       *
 *   published by the Free Software Foundation; either version 2 of the    *
 *   License, or (at your option) any later version.                       *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Library General Public     *
 *   License along with this program; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA.         *
 ***************************************************************************/

#ifndef KBSMOLECULEMODEL_H
#define KBSMOLECULEMODEL_H

#include <QDomDocument>
#include <QHash>
#include <QStringList>
#include <QTextStream>

#include <kbsmolbiodata.h>

#include <kbsmolbio_export.h>

class KBSMOLBIO_EXPORT KBSMoleculeRenderer
{
  public:
    virtual ~KBSMoleculeRenderer() {}
    
    virtual void setColor(QColor color) = 0;
    virtual void drawBall(double *v, double radius) = 0;
    virtual void drawLine(double *v1, double *v2, double thickness=0.0) = 0;
    virtual void drawQuadrangle(double *v1, double *v2, double *v3, double *v4) = 0;
  
  protected:
    KBSMoleculeRenderer() {};
};

class KBSMOLBIO_EXPORT KBSVRMLRenderer : public KBSMoleculeRenderer
{
  public:
    KBSVRMLRenderer(const QString &fileName);
    virtual ~KBSVRMLRenderer();
    
    virtual bool isOpen() const;
    virtual void close();
    
    virtual void setColor(QColor color);
    virtual void drawBall(double *v, double radius);
    virtual void drawLine(double *v1, double *v2, double thickness=0.0);
    virtual void drawQuadrangle(double *v1, double *v2, double *v3, double *v4);
  
  protected:
    QString m_color;
    QStringList m_lineCoords, m_lineColors, m_faceCoords, m_faceColors;
    QTextStream m_text;
  
  private:
    bool m_open;
};

class KBSMOLBIO_EXPORT KBSX3DRenderer : public KBSMoleculeRenderer
{
  public:
    KBSX3DRenderer(const QString &fileName);
    virtual ~KBSX3DRenderer();
    
    virtual bool isOpen() const;
    virtual void close();
    
    virtual void setColor(QColor color);
    virtual void drawBall(double *v, double radius);
    virtual void drawLine(double *v1, double *v2, double thickness=0.0);
    virtual void drawQuadrangle(double *v1, double *v2, double *v3, double *v4);
  
  protected:
    QString m_color;
    QStringList m_lineCoords, m_lineColors, m_faceCoords, m_faceColors;
    QDomDocument m_doc;
    QDomElement m_group;
    QTextStream m_text;
  
  private:
    bool m_open;
};

class KBSMOLBIO_EXPORT KBSMoleculeModel : public QObject
{
  Q_OBJECT
  public:
    KBSMoleculeModel(QObject *parent=0);
    virtual ~KBSMoleculeModel();
  
    enum Style { Backbone, Spline,
                 Wireframe, Sticks, Spacefill, BallAndStick };
    virtual Style style() const;
    virtual void setStyle(Style style);
    virtual bool isSupportedStyle(Style style) const;
    
    enum Coloring { Monochrome, Group, Shapely, CPK, Structure };
    virtual Coloring coloring() const;
    virtual void setColoring(Coloring coloring);
    virtual bool isSupportedColoring(Coloring coloring) const;
    
    virtual void setChain(const KBSProteinChain &chain);
    virtual void setFASTA(const KBSProteinFASTA &fasta);
    virtual void setPDB(const KBSProteinPDB &pdb);
    
    virtual void rotateData(int dx, int dy);
  
    virtual bool exportVRML(const QString &fileName) const;
    virtual bool exportX3D(const QString &fileName) const;
    
    friend class KBSMoleculeView;
  
  signals:
    void styleChanged();
    void coloringChanged();
    void dataChanged();
  
  protected:
    virtual void render(KBSMoleculeRenderer *renderer) const;
  
  private:
    void interpolateBackbone();
  
  protected:
    unsigned m_maxSeq;
    QList<unsigned> m_group;
    QList<KBSProteinAtom> m_backbone;
    QList<KBSAminoAcid> m_sequence;
    QList<KBSAtomPDB> m_atom;
    
    double *m_backboneCoords, *m_atomCoords;
    
    static const unsigned s_divisions;
  
  private:
    Style m_style;
    Coloring m_coloring;
};

#endif
